function [proj_set,weight,backproj_set] = make_homo_projection(rot_mat,rot_inv_mat,loc,mode)
% calculate projection correspondence 
% proj_set: from sharp to blurry
% backproj_set: from blurry to sharp

num_loc = size(loc,1);
num = size(rot_inv_mat,1);
proj_set = cell(1,num_loc);
backproj_set = cell(1,num_loc);
weight = [];
backweight = [];

for i=1:num_loc
    temp = zeros(9,3);
    temp(1,1) = loc(i,1);
    temp(4,1) = loc(i,2);
    temp(7,1) = 1;
    temp(2,2) = loc(i,1);
    temp(5,2) = loc(i,2);
    temp(8,2) = 1;
    temp(3,3) = loc(i,1);
    temp(6,3) = loc(i,2);
    temp(9,3) = 1;
    proj_temp = rot_inv_mat*temp;
    denom = proj_temp(:,3)+1e-10;
    
    location = proj_temp(:,1:2)./repmat(denom,[1,2])-repmat([loc(i,1),loc(i,2)],[num,1]);
    location_floor = floor(location);
    if(strcmp(mode,'round'))
        proj_set{1,i} = round(location);
        weight = [weight;1];
    elseif(strcmp(mode,'bilinear'))
        dif = location - location_floor;
        scal = (1+dif(:,1)./(1-dif(:,1)+0.001)).*(1+dif(:,2)./(1-dif(:,2)+0.001));
        scal = 1./scal;
%         proj_set{1,i} = [location_floor(1),location_floor(2);location_floor(1)+1,location_floor(2);location_floor(1),location_floor(2)+1;location_floor(1)+1,location_floor(2)+1];
        proj_set{1,i} = location_floor;
        weight = [weight;scal,scal.*dif(:,1)./(1-dif(:,1)),scal.*dif(:,2)./(1-dif(:,2)),scal.*dif(:,1)./(1-dif(:,1)).*dif(:,2)./(1-dif(:,2))];
    end
    
    %backprojection
    backproj_temp = rot_mat*temp;
    denom = backproj_temp(:,3)+1e-10;
    
    location = backproj_temp(:,1:2)./repmat(denom,[1,2])-repmat([loc(i,1),loc(i,2)],[num,1]);
    location_floor = floor(location);
    if(strcmp(mode,'round'))
        backproj_set{1,i} = round(location);
        backweight = [backweight;1];
    elseif(strcmp(mode,'bilinear'))
        dif = location - location_floor;
        scal = (1+dif(:,1)./(1-dif(:,1)+0.001)).*(1+dif(:,2)./(1-dif(:,2)+0.001));
        scal = 1./scal;
%         proj_set{1,i} = [location_floor(1),location_floor(2);location_floor(1)+1,location_floor(2);location_floor(1),location_floor(2)+1;location_floor(1)+1,location_floor(2)+1];
        backproj_set{1,i} = location_floor;
        backweight = [backweight;scal,scal.*dif(:,1)./(1-dif(:,1)),scal.*dif(:,2)./(1-dif(:,2)),scal.*dif(:,1)./(1-dif(:,1)).*dif(:,2)./(1-dif(:,2))];
    end
end
    