function [pose_set,pose_inv_set,pose_ind,w] = pose_perturbation(pose_set_old,pose_inv_set_old,pose_ind_old,w_old,grid_x,grid_y,grid_z,proj_set,proj_weight,im_size,rot_inv_mat,mode,num_cut,ind_used)
% This function add perturbation to possible camera poses.
% 

n = length(grid_x)*length(grid_y)*length(grid_z);
prob_set = [];
thresh = 0.02; % to determine each point
len_x = length(grid_x);
len_y = length(grid_y);
len_z = length(grid_z);
len_set = length(w_old);

%% determine which poses to use
if(strcmp(mode,'Gaussian'))
    mu = [0,0,0];
    sigma = [2,0.5,0.5;0.5,2,0.5;0.5,0.5,3];
    sample = round(mgd(1000,3,mu,sigma));
    sample_count = 1;
    
    [temp_value,temp_sortind] = sort(w_old,'descend');
    num_cut = sum((w_old<temp_value(1)*0.05));
    fprintf('%d ',num_cut);
    
    ind_temp = pose_ind_old(temp_sortind(1:end-num_cut));
    [pose_sub1,pose_sub2,pose_sub3] = ind2sub([len_x,len_y,len_z],ind_temp);
    pose_sub = [pose_sub1,pose_sub2,pose_sub3];
    
    pose_ind = ind_temp;
    [pose_sub1,pose_sub2,pose_sub3] = ind2sub([len_x,len_y,len_z],ind_used);
    used_sub = [pose_sub1,pose_sub2,pose_sub3];
    used_sub = pose_sub;
    
    nn = length(ind_temp);
    count = 0;
    while(count<num_cut)
        add_ind = randi([1,nn]);
        sample_sub = pose_sub(add_ind,:)+sample(sample_count,:);
        if((sample_sub(1)>0) && (sample_sub(1)<=len_x) && (sample_sub(2)>0) ...
                && (sample_sub(2)<=len_y) && (sample_sub(3)>0) && (sample_sub(3)<= len_z)) 
            temp = abs(used_sub-repmat(sample_sub,[size(used_sub,1),1]));
            temp = sum(temp,2);

            if(sum(temp==0)==0)
                ind_new = sub2ind([len_x,len_y,len_z],sample_sub(1),sample_sub(2),sample_sub(3));
                pose_ind = [pose_ind;ind_new];
                used_sub = [used_sub;sample_sub];
                count = count +1;
            end
        end
        sample_count = sample_count+1;
        if (sample_count>1000)
            sample = round(mgd(1000,3,mu,sigma));
            sample_count = 1;
        end
    end    
else
end

w = zeros(size(w_old));
w(1:end-num_cut) = w_old(temp_sortind(1:end-num_cut));
%% generate pose basis and weights
temp_rot = rot_inv_mat(pose_ind,:);
num_ind = length(pose_ind);
pose_set = cell(1,num_ind);
pose_inv_set = cell(1,num_ind);
[Y,X] = meshgrid(1:im_size(2),1:im_size(1));
row_ind = cell(1,num_ind);
col_ind = cell(1,num_ind);
value_ind = cell(1,num_ind);
dim_im = im_size(1)*im_size(2);

    temp1 = zeros(9,dim_im);
    temp2 = zeros(9,dim_im);
    temp3 = zeros(9,dim_im);
    
    X_row = X(:)';
    Y_row = Y(:)';
    temp1(1,:) = X_row;    
    temp2(2,:) = X_row;
    temp3(3,:) = X_row;
    temp1(4,:) = Y_row;    
    temp2(5,:) = Y_row;
    temp3(6,:) = Y_row;
    temp1(7,:) = ones(1,dim_im);
    temp2(8,:) = ones(1,dim_im);
    temp3(9,:) = ones(1,dim_im);
    
    proj1 = temp_rot*temp1;
    proj2 = temp_rot*temp2;
    proj3 = temp_rot*temp3;

    location1 = proj1./proj3;
    location1_floor = floor(location1);
    location2 = proj2./proj3;
    location2_floor = floor(location2);    
    dif1 = location1 - location1_floor;
    dif2 = location2 - location2_floor;
    scal = (1+dif1./(1-dif1)).*(1+dif2./(1-dif2));
    scal = 1./scal;
    weight = [scal,scal.*dif1./(1-dif1),scal.*dif2./(1-dif2),scal.*dif1./(1-dif1).*dif2./(1-dif2)];
    
    temp_loc = (location2_floor-1)*im_size(1)+location1_floor;
    temp_col_ind = [temp_loc,temp_loc+1,temp_loc+im_size(1),temp_loc+im_size(1)+1];
    temp_row_ind = [1:dim_im,1:dim_im,1:dim_im,1:dim_im];
    temp_value_ind = weight;
    
    temp_inv_loc = (2*repmat(Y_row,[num_ind,1])-location2_floor-1)*im_size(1)+(2*repmat(X_row,[num_ind,1])-location1_floor);
    temp_inv_col_ind = [temp_inv_loc,temp_inv_loc-1,temp_inv_loc-im_size(1),temp_inv_loc-im_size(1)-1];
    
for i =1:(num_ind-num_cut)
    pose_set{1,i} = pose_set_old{1,temp_sortind(i)};
    pose_inv_set{1,i} = pose_inv_set_old{1,temp_sortind(i)};
end
    
for i =(num_ind-num_cut+1):num_ind
    temp1 = temp_row_ind;
    temp2 = temp_col_ind(i,:);
    temp3 = temp_value_ind(i,:);
    ind1 = find(temp2>0);
    ind2 = find(temp2>dim_im);
    final_ind = setxor(ind1,ind2);
    row_ind{1,i} = temp1(final_ind);
    col_ind{1,i} = temp2(final_ind);
    value_ind{1,i} = temp3(final_ind);
    pose_set{1,i} = sparse(temp1(final_ind),temp2(final_ind),temp3(final_ind),dim_im,dim_im);

    temp2 = temp_inv_col_ind(i,:);
    ind1 = find(temp2>0);
    ind2 = find(temp2>dim_im);
    final_ind = setxor(ind1,ind2);
    pose_inv_set{1,i} = sparse(temp1(final_ind),temp2(final_ind),temp3(final_ind),dim_im,dim_im);

end

    
