function [pose_set,pose_inv_set,pose_ind,w0] = pose_restrict(row_k,col_k,grid_x,grid_y,grid_z,kernel_set,proj_set,proj_weight,loc_set,im_size,rot_inv_mat)

% This function initialize the camera motion using local blur kernels and
% compute the pose basis and their weight

half_row = (row_k-1)/2;
half_col = (col_k-1)/2;
num_loc = size(loc_set,1);
n = length(grid_x)*length(grid_y)*length(grid_z);
prob_set = [];
thresh = 0.01; % to determine each point

%% determine which poses to use
if(1)
for i =1:num_loc
    temp_loc = loc_set(i,:);
    temp_proj = proj_set{1,i};
    temp_proj = temp_proj + repmat([half_row+1,half_col+1],[size(temp_proj,1),1]);
    temp_k = rot90(kernel_set{1,i},2);

    ind = (temp_proj(:,2)-1)*row_k+temp_proj(:,1);
    ind = [ind,ind+1,ind+col_k,ind+col_k+1];
    temp_value = zeros(size(ind));
    temp_value(temp_k(ind)>thresh)=1;
    prob_set = [prob_set,sum(temp_value,2)];
end

prob = sum(prob_set,2)/4/num_loc;
[temp1,temp2] = sort(prob,'descend');
bar = 60;
pose_ind = temp2(1:bar);
w0 = prob(pose_ind);
w0 = w0./sum(w0);
else 
for i =1:num_loc
    temp_loc = loc_set(i,:);
    temp_proj = proj_set{1,i};
    %deal with kernel range
    tt = max(abs(temp_proj(:)));
    if(tt>half_row)
        pad_row = tt-half_row;
    else
        pad_row = 0;
    end
    if(tt>half_col)
        pad_col = tt-half_col;
    else
        pad_col = 0;
    end    
    
    temp_k = zeros(row_k+2*pad_row,col_k+2*pad_col);
    temp_proj = temp_proj + repmat([pad_row+half_row+1,pad_col+half_col+1],[size(temp_proj,1),1]);
    temp_k(pad_row+1:pad_row+row_k,pad_col+1:pad_col+col_k) = rot90(kernel_set{1,i},2);
%     temp_k(pad_row+1:pad_row+row_k,pad_col+1:pad_col+col_k) = kernel_set{1,i};
    
    ind = (temp_proj(:,2)-1)*(row_k+2*pad_row)+temp_proj(:,1);
    temp_value = temp_k(ind);
    prob_set = [prob_set,temp_value];
end

prob = sum(prob_set,2)/sum(prob_set(:));
[temp1,temp2] = sort(prob,'descend');
bar = 60;
pose_ind = temp2(1:bar);
% pose_ind = [113];
w0 = prob(pose_ind);
end

%% generate pose basis and weights
temp_rot = rot_inv_mat(pose_ind,:);
num_ind = length(pose_ind);
pose_set = cell(1,num_ind);
pose_inv_set = cell(1,num_ind);
[Y,X] = meshgrid(1:im_size(2),1:im_size(1));
row_ind = cell(1,num_ind);
col_ind = cell(1,num_ind);
value_ind = cell(1,num_ind);
dim_im = im_size(1)*im_size(2);

    temp1 = zeros(9,dim_im);
    temp2 = zeros(9,dim_im);
    temp3 = zeros(9,dim_im);
    
    X_row = X(:)';
    Y_row = Y(:)';
    temp1(1,:) = X_row;    
    temp2(2,:) = X_row;
    temp3(3,:) = X_row;
    temp1(4,:) = Y_row;    
    temp2(5,:) = Y_row;
    temp3(6,:) = Y_row;
    temp1(7,:) = ones(1,dim_im);
    temp2(8,:) = ones(1,dim_im);
    temp3(9,:) = ones(1,dim_im);
    
    proj1 = temp_rot*temp1;
    proj2 = temp_rot*temp2;
    proj3 = temp_rot*temp3;
    clear temp1;clear temp2;clear temp3;

    location1 = proj1./proj3;
    location1_floor = floor(location1);
    location2 = proj2./proj3;
    location2_floor = floor(location2);    
    dif1 = location1 - location1_floor;
    dif2 = location2 - location2_floor;
    scal = (1+dif1./(1-dif1)).*(1+dif2./(1-dif2));
    scal = 1./scal;
    weight = [scal,scal.*dif1./(1-dif1),scal.*dif2./(1-dif2),scal.*dif1./(1-dif1).*dif2./(1-dif2)];
    
    temp_loc = (location2_floor-1)*im_size(1)+location1_floor;
    temp_col_ind = [temp_loc,temp_loc+1,temp_loc+im_size(1),temp_loc+im_size(1)+1];
    temp_row_ind = [1:dim_im,1:dim_im,1:dim_im,1:dim_im];
    temp_value_ind = weight;
    
    temp_inv_loc = (2*repmat(Y_row,[num_ind,1])-location2_floor-1)*im_size(1)+(2*repmat(X_row,[num_ind,1])-location1_floor);
    temp_inv_col_ind = [temp_inv_loc,temp_inv_loc-1,temp_inv_loc-im_size(1),temp_inv_loc-im_size(1)-1];
    
    
for i =1:num_ind
    temp1 = temp_row_ind;
    temp2 = temp_col_ind(i,:);
    temp3 = temp_value_ind(i,:);
    ind1 = find(temp2>0);
    ind2 = find(temp2>dim_im);
    final_ind = setxor(ind1,ind2);
    row_ind{1,i} = temp1(final_ind);
    col_ind{1,i} = temp2(final_ind);
    value_ind{1,i} = temp3(final_ind);
    pose_set{1,i} = sparse(temp1(final_ind),temp2(final_ind),temp3(final_ind),dim_im,dim_im);

    temp2 = temp_inv_col_ind(i,:);
    ind1 = find(temp2>0);
    ind2 = find(temp2>dim_im);
    final_ind = setxor(ind1,ind2);
    pose_inv_set{1,i} = sparse(temp1(final_ind),temp2(final_ind),temp3(final_ind),dim_im,dim_im);
end
    
