function [w] = pose_weight_estimatation(Px,Py,B,w0,pose_set)
% The function is used to estimate camera motion given predicted gradient
% maps.
% 
% Required input
% Px,Py: predicted gradient maps
% B: blurry image
% omega: weights for partial derivative
% beta: Tihkonov parameter
% 

omega = [25,12.5]; % weight for derivative

[im_row,im_col] = size(B);
[Bx,By] = gradient(B,1);
[Bxx,Bxy] = gradient(Bx,1);
[Bxy,Byy] = gradient(By,1);

[Pxx,Pxy1] = gradient(Px,1);
[Pxy2,Pyy] = gradient(Py,1);
Pxy = (Pxy1+Pxy2)/2;

%% build data for optimization
n_basis = size(pose_set,2);
dim_im = size(pose_set{1,1},1);
% A = zeros(patch_size(1)*patch_size(2)*n_loc*5,n_proj);
% b = zeros(patch_size(1)*patch_size(2)*n_loc*5,1);
A = [];
b = [];


bx_temp = Bx;
by_temp = By;
bxx_temp = Bxx;
bxy_temp = Bxy;
byy_temp = Byy;
b = [omega(1)*bx_temp(:);omega(1)*by_temp(:);omega(2)*bxx_temp(:);omega(2)*bxy_temp(:);omega(2)*byy_temp(:)];
    
% K * \partial L
if(1)
    for i=1:n_basis  
        Lx_basis_temp = pose_set{1,i}*Px(:);
        Ly_basis_temp = pose_set{1,i}*Py(:);
        [Lxx,Lxy1] = gradient(reshape(Lx_basis_temp,size(B)),1);
        [Lxy2,Lyy] = gradient(reshape(Ly_basis_temp,size(B)),1);
        Lxy = (Lxy1+Lxy2).*0.5;
        
        Lxx_basis_temp = Lxx(:);
        Lxy_basis_temp = Lxy(:);
        Lyy_basis_temp = Lyy(:);
        temp = [omega(1)*Lx_basis_temp;omega(1)*Ly_basis_temp;omega(2)*Lxx_basis_temp;omega(2)*Lxy_basis_temp;omega(2)*Lyy_basis_temp];
        A = [A,temp];
    end

end
% \partial (K*L)
if(0)
    for i=1:n_basis  
        Lx_basis_temp = pose_set{1,i}*Px(:);
        Ly_basis_temp = pose_set{1,i}*Py(:);
        Lxx_basis_temp = pose_set{1,i}*Pxx(:);
        Lxy_basis_temp = pose_set{1,i}*Pxy(:);
        Lyy_basis_temp = pose_set{1,i}*Pyy(:);
        temp = [omega(1)*Lx_basis_temp;omega(1)*Ly_basis_temp;omega(2)*Lxx_basis_temp;omega(2)*Lxy_basis_temp;omega(2)*Lyy_basis_temp];
        A = [A,temp];
    end
end
    temp = A(1:dim_im,:);
    ATA = temp'*temp*omega(1);
    ATb = temp'*Bx(:)*omega(1);
    temp = A(dim_im+1:dim_im*2,:);
    ATA = ATA+temp'*temp*omega(1);
    ATb = ATb+temp'*By(:)*omega(1);
    temp = A(2*dim_im+1:dim_im*3,:);
    ATA = ATA+temp'*temp*omega(2);
    ATb = ATb+temp'*Bxx(:)*omega(2);
    temp = A(3*dim_im+1:dim_im*4,:);
    ATA = ATA+temp'*temp*omega(2);
    ATb = ATb+temp'*Bxy(:)*omega(2);
    temp = A(4*dim_im+1:dim_im*5,:);
    ATA = ATA+temp'*temp*omega(2);
    ATb = ATb+temp'*Byy(:)*omega(2);

clear bx_temp by_temp bxx_temp bxy_temp byy_temp;
clear Lx_basis_temp Ly_basis_temp Lxx_basis_temp Lxy_basis_temp Lyy_basis_temp;
clear temp;

beta =1;
if(0)
    w = linearcg_w(ATA,ATb,beta,w0);
else
%     A_in = ATA + beta*eye(size(ATA,1));
%     w = L1QP_FeatureSign_yang(1,A_in,-ATb);
%     w = l1_ls_nonneg(A,b,beta);
%     w = LassoProjection(A,b,beta);
    
    H = ATA + beta*eye(size(ATA,1));
    f = -ATb;
%     L = [ones(1,length(w0));-ones(1,length(w0))];
%     k = [1;-0.5];
    L = [ones(1,length(w0))];
    k = [1];
    A = [];
    b = [];
    l = zeros(length(w0),1);
    u = [];
    
    [w,err,lm] = qpip(H,f,L,k,A,b,l,u,0,0,0);
end
w(w<max(w(:))*0.01)=0;
% sum(w)
w = w./sum(w(:));
end

function k = linearcg_w(ATA,ATb,beta,k0,tol,maxit)
[row,col] = size(ATA);
A = 2*(ATA+beta*eye(row));
b = 2*ATb;

if ~exist('tol','var') || isempty(tol) tol = 1e-6; end;
if ~exist('maxit','var') || isempty(maxit) maxit = 300; end;

% ---------- Initialize -------------------------
k = k0(:);				           % Current iterate
r = A*k - b;
p = -r;                        % Current conjugate
iter = 0;

% ---------- Begin iteration ------------------------
while ((norm(r) > tol) && (iter < maxit))
  
    rr = r' * r;
    pAp = p' * A * p;
    alpha = rr / pAp;    % Compute the step length
    
    k = k + alpha * p;       % Update x
     
    r = r + alpha * A * p;   % Update residual 
    
    rr_new = r' * r;
    rho = rr_new / rr;  % Compute beta
    
    p = rho * p - r;        % Update cg vector 
    iter = iter +1;
end

end
